/* These typedefs are from the libodbc 2.2.0 include files.*/

/* typedefs are from sqltypes.h  */

typedef signed short int   SQLSMALLINT;
typedef unsigned short     SQLUSMALLINT;
typedef int                SQLINTEGER;
typedef unsigned int       SQLUINTEGER;
typedef unsigned char      SQLCHAR;
typedef void *             SQLHANDLE;
typedef void *             SQLPOINTER;
typedef SQLSMALLINT        SQLRETURN;
typedef SQLHANDLE          SQLHENV;
typedef SQLHANDLE          SQLHDBC;
typedef SQLHANDLE          SQLHSTMT;
typedef SQLHANDLE          SQLHDESC;
typedef int                SQLLEN;
typedef unsigned int       SQLULEN;

/* typedefs from cursorlib.h */ 
typedef	void *  DMHDBC ;

/* typedefs from datamanager.h */
typedef void *  CLHSTMT ;



